package com.gic.evaluate.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.evaluate.constant.Constants;
import com.gic.evaluate.constant.ReplyEnum;
import com.gic.evaluate.dto.EvaluateDTO;
import com.gic.evaluate.dto.EvaluateQuickDTO;
import com.gic.evaluate.dto.EvaluateReplyDTO;
import com.gic.evaluate.dto.UserReplyDTO;
import com.gic.evaluate.entity.*;
import com.gic.evaluate.qo.EvaluateQO;
import com.gic.evaluate.service.*;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-12-27 9:50
 */
@Service("evaluateManageApiService")
public class EvaluateManageApiServiceImpl implements EvaluateManageApiService {

    @Autowired
    private EvaluateService evaluateService;
    @Autowired
    private EvaluateReplyService evaluateReplyService;
    @Autowired
    private EvaluateQuickService evaluateQuickService;
    @Autowired
    private EvaluateImgService evaluateImgService;

    @Override
    public ServiceResponse<Page<EvaluateDTO>> listEvaluateForManage(EvaluateQO evaluateQO) {
        com.github.pagehelper.Page<TabEvaluate> evaluateList = evaluateService.listEvaluate(evaluateQO);
        Page<EvaluateDTO> page = PageHelperUtils.changePageHelperToCurrentPage(evaluateList, EvaluateDTO.class);
        List<EvaluateDTO> result = page.getResult();
        if (CollectionUtils.isNotEmpty(result)) {
            List<Integer> storeIdList = result.stream().map(EvaluateDTO::getStoreId).collect(Collectors.toList());
            Set<Integer> storeIdHasTopHot = this.getStoreIdHasTopHot(evaluateQO.getEnterpriseId(), storeIdList);
            for (EvaluateDTO evaluateDTO : result) {
                evaluateDTO.setHasTopHot(storeIdHasTopHot.contains(evaluateDTO.getStoreId()));
            }
        }
        return EnterpriseServiceResponse.success(page);
    }

    @Override
    public ServiceResponse<EvaluateDTO> getEvaluate(Integer enterpriseId, Integer evaluateId) {
        TabEvaluate evaluate = evaluateService.getEvaluate(enterpriseId, evaluateId);
        if (evaluate == null) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }
        EvaluateDTO evaluateDTO = EntityUtil.changeEntityByJSON(EvaluateDTO.class, evaluate);
        List<TabEvaluateReply> tabReplyList = evaluateReplyService.listEvaluateReply(enterpriseId, evaluateId);
        List<EvaluateReplyDTO> replyDTOList = EntityUtil.changeEntityListByJSON(EvaluateReplyDTO.class, tabReplyList);
        for (EvaluateReplyDTO evaluateReplyDTO : replyDTOList) {
            List<TabEvaluateImg> tabEvaluateImgs = evaluateImgService.listImageByEvaluateReplyId(evaluateReplyDTO.getEvaluateReplyId());
            List<String> replyImgList = Optional.ofNullable(tabEvaluateImgs).orElse(Collections.emptyList()).stream().map(TabEvaluateImg::getImgUrl).collect(Collectors.toList());
            evaluateReplyDTO.setImgList(replyImgList);
        }
        evaluateDTO.setReplyList(replyDTOList);
        return EnterpriseServiceResponse.success(evaluateDTO);
    }

    @Override
    public ServiceResponse<Void> updateShowStatus(Integer enterpriseId, List<Integer> evaluateId, Integer showStatus) {
        if (CollectionUtils.isNotEmpty(evaluateId)) {
            if (evaluateId.size() == 1 && evaluateId.get(0) == -1) {
                // 如果传了-1  就是修改全部
                evaluateId = Collections.emptyList();
            }
            this.evaluateService.updateShowStatus(enterpriseId, evaluateId, showStatus);
        }
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> updateReplyShowStatus(Integer enterpriseId, Integer evaluateReplyId, Integer showStatus) {
        this.evaluateReplyService.updateReplyShowStatus(enterpriseId, evaluateReplyId, showStatus);
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> updateTopHotStatus(Integer enterpriseId, Integer evaluateId, Integer topHotStatus) {
        if (Constants.OPEN.equals(topHotStatus)) {
            TabEvaluate evaluate = this.evaluateService.getEvaluate(enterpriseId, evaluateId);
            Integer oldEvaluateId = this.evaluateService.getStoreTopHot(enterpriseId, evaluate.getStoreId());
            this.evaluateService.updateTopHotStatus(enterpriseId, oldEvaluateId, Constants.CLOSE);
        }
        this.evaluateService.updateTopHotStatus(enterpriseId, evaluateId, topHotStatus);
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> updateMaliciousStatus(Integer enterpriseId, Integer evaluateId, Integer maliciousStatus) {
        this.evaluateService.updateMaliciousStatus(enterpriseId, evaluateId, maliciousStatus);
        if (Constants.OPEN.equals(maliciousStatus)) {
            this.evaluateService.updateShowStatus(enterpriseId, Collections.singletonList(evaluateId), Constants.CLOSE);
        } else if (Constants.CLOSE.equals(maliciousStatus)) {
            this.evaluateService.updateShowStatus(enterpriseId, Collections.singletonList(evaluateId), Constants.OPEN);
        }
        return EnterpriseServiceResponse.success();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ServiceResponse<Void> replyEvaluate(UserReplyDTO userReplyDTO) {
        if (userReplyDTO.getEvaluateIds() != null) {
            List<Integer> evaluateIdList;
            if (userReplyDTO.getEvaluateIds().length == 1 && userReplyDTO.getEvaluateIds()[0] == -1) {
                // 如果传了-1  就是修改全部
                evaluateIdList = evaluateService.listAllEvaluateId(userReplyDTO.getEnterpriseId());
            } else {
                evaluateIdList = Arrays.asList(userReplyDTO.getEvaluateIds());
            }
            for (Integer evaluateId : evaluateIdList) {
                userReplyDTO.setEvaluateId(evaluateId);
                this.evaluateReplyService.saveUserReplyEvaluate(userReplyDTO);
                EvaluateDTO evaluateDTO = new EvaluateDTO();
                evaluateDTO.setEvaluateId(evaluateId);
                evaluateDTO.setReplyStatus(ReplyEnum.REPLIED.getCode());
                this.evaluateService.updateEvaluate(evaluateDTO);
            }
        }
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> editReplyEvaluate(UserReplyDTO userReplyDTO) {
        evaluateReplyService.update(userReplyDTO);
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> delReply(Integer evaluateReplyId) {
        this.evaluateReplyService.delReply(evaluateReplyId);
        return EnterpriseServiceResponse.success();
    }

    private Set<Integer> getStoreIdHasTopHot(Integer enterpriseId, List<Integer> storeIdList) {
        return this.evaluateService.getStoreIdHasTopHot(enterpriseId, storeIdList);
    }

    @Override
    public ServiceResponse<List<EvaluateQuickDTO>> listEvaluateQuickReply(Integer enterpriseId, Integer quickReplyType) {
        List<TabEvaluateQuick> evaluateQuickList = this.evaluateQuickService.listEvaluateQuick(enterpriseId, quickReplyType);
        return EnterpriseServiceResponse.success(EntityUtil.changeEntityListByJSON(EvaluateQuickDTO.class, evaluateQuickList));
    }

    @Override
    public ServiceResponse<Integer> countEvaluate(EvaluateQO evaluateQO) {
        Integer count = evaluateService.countEvaluate(evaluateQO);
        return EnterpriseServiceResponse.success(count);
    }

}
